{========================================================================}
{=================      TVicHW32  for Kylix             =================}
{======         Copyright (c) 1997-2003 Victor I.Ishikeev           =====}
{========================================================================}
{==========             mailto:tools@entechtaiwan.com             =======}
{========================================================================}

unit HW_32;

{$T-}

{--------}  interface    {---------}

uses SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
     Libc, KernelIoctl, IdGlobal, HW_Types;

type TInterruptHandler = procedure (IrqNumber : Word) of object;

type

  TVicHw32 = class(TComponent)

  private

    hDRV             : Integer;

    fOpenDrive       : Boolean;

    fHardAccess      : Boolean;
    fIoPrivLevel     : Longword;

    fMapped         : Boolean;
    fMappedArea     : Longword;
    fMappedSize     : Longword;
    fKernelArea     : Longword;

    fOnHwInterrupt   : TInterruptHandler;
    fHandled         : Longword;

    fMasked          : Boolean;
    fIrqNumber       : Byte;

    fLPTIRQEnabled   : Boolean;
    fLptIrqForced    : Byte;
    fLPTReadMode     : Boolean;
    fLPTAddresses    : array[1..4] of Word;
    fLPTBasePort     : Word;
    fDataWritePorts  : array[0..2] of Byte;
    fDataReadPorts   : array[0..2] of Byte;
    fLPTNumber       : Byte;
    fLPTs            : Byte;


    function         fGetPortB(PortAddr: word) : Byte;
    procedure        fSetPortB(PortAddr: word; nNewValue : Byte);
    function         fGetPortW(PortAddr: word) : Word;
    procedure        fSetPortW(PortAddr: word; nNewValue : Word);
    function         fGetPortL(PortAddr: word) : Longword;
    procedure        fSetPortL(PortAddr: word; nNewValue : Longword);
    function         fMapPhysToLinear(PhAddr : Longword; PhSize : Longword) : Pointer;
    procedure        fUnmapMemory();

    function         fGetMemB(BaseAddress:Longword; Offset : Longword) : Byte;
    procedure        fSetMemB(BaseAddress:Longword; Offset : Longword; pval : Byte);
    function         fGetMemW(BaseAddress:Longword; Offset : Longword) : Word;
    procedure        fSetMemW(BaseAddress:Longword; Offset : Longword; pval : Word);
    function         fGetMemL(BaseAddress:Longword; Offset : Longword) : Longword;
    procedure        fSetMemL(BaseAddress:Longword; Offset : Longword; pval : Longword);

    procedure        fSetHardAccess(Parm : Boolean);
    function         fSetIoPermission( PortAddr : word): Boolean;
    procedure        fResetAllPermissions();

    procedure        fReadMappedMemory (MemRec: pMemRec);
    procedure        fWriteMappedMemory(MemRec: pMemRec);

    procedure        fSetLPTIRQ(nNewValue  : Boolean);
    procedure        fSetLPTNumber(nNewValue  : Byte);
    function         fGetPin(nPin : Byte) : Boolean;
    procedure        fSetPin(nPin : Byte; bNewValue : Boolean);
    procedure        fSetLPTReadMode(readmode : Boolean);

    function         fGetLPTAckwl : Boolean;
    function         fGetLPTBusy : Boolean;
    function         fGetLPTPaperEnd : Boolean;
    function         fGetLPTSlct : Boolean;
    function         fGetLPTError : Boolean;

  public

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    procedure  OpenDriver;
    procedure  CloseDriver;
    
    procedure   ReadPortFIFO  ( PortAddr: word; Counter:Longword; Buffer: Pointer);
    procedure   WritePortFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
    procedure   ReadPortWFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
    procedure   WritePortWFIFO( PortAddr: word; Counter:Longword; Buffer: Pointer);
    procedure   ReadPortLFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
    procedure   WritePortLFIFO( PortAddr: word; Counter:Longword; Buffer: Pointer);

    procedure   ReadPhysicalMemory (PhysAddress: Longword; Offset: Longword; Counter: Longword; Buffer: Pointer);
    procedure   WritePhysicalMemory(PhysAddress: Longword; Offset: LongWord; Counter: Longword; Buffer: Pointer);

    procedure   UnmaskInterrupt(IrqNumber : Word);
    procedure   MaskInterrupt();
    function    InterruptCounter: Longword;
    function    GetHandledCounter: Longword;

    function    PciDeviceInfo(bus,dev,func : Word; Info : pPciCfg) : Boolean;
    function    PciGetByte(bus,dev,func: Word; Offset: Word) : Byte;
    function    PciGetWord(bus,dev,func: Word; Offset: Word) : Word;
    function    PciGetDword(bus,dev,func: Word; Offset: Word) : Longword;

    procedure   PciSetByte(bus,dev,func: Word; Offset: Word; nByte:Byte);
    procedure   PciSetWord(bus,dev,func: Word; Offset: Word; nWord:Word);
    procedure   PciSetDword(bus,dev,func: Word; Offset: Word; nDWord:DWord);

    procedure   LPTStrobe;
    procedure   LPTAutofd(Flag : Boolean);
    procedure   LPTInit;
    procedure   LPTSlctIn;


    function    GetDebugCode: Longword;


    property    Port[Index:word]  : Byte  read fGetPortB  write fSetPortB;
    property    PortW[Index:word] : Word  read fGetPortW write fSetPortW;
    property    PortL[Index:word] : Longword read fGetPortL write fSetPortL;

    property    Mem[BaseAddress:Longword; Offset:Longword]  : Byte  read fGetMemB write fSetMemB;
    property    MemW[BaseAddress:Longword; Offset:Longword] : Word  read fGetMemW write fSetMemW;
    property    MemL[BaseAddress:Longword; Offset:Longword] : Longword read fGetMemL write fSetMemL;

    property    IrqMasked     : Boolean read fMasked;
    property    ActiveHW      : Boolean read fOpenDrive;

    property    LPTIRQEnabled: Boolean read fLPTIRQEnabled write fSetLPTIRQ;
    property    Pin[Index:Byte] : Boolean read fGetPin write fSetPin;
    property    LPTAckwl     : Boolean read fGetLPTAckwl;
    property    LPTBusy      : Boolean read fGetLPTBusy;
    property    LPTPaperEnd  : Boolean read fGetLPTPaperEnd;
    property    LPTSlct      : Boolean read fGetLPTSlct;
    property    LPTError     : Boolean read fGetLPTError;
    property    LPTNumPorts  : Byte    read fLPTs;
    property    LPTBasePort  : Word   read fLPTBasePort;
    property    LPTReadMode : Boolean read fLPTReadMode write fSetLPTReadMode;

  published

    property    OnHwInterrupt : TInterruptHandler read fOnHwInterrupt write fOnHwInterrupt;
    property    HardAccess    : Boolean read fHardAccess write fSetHardAccess default TRUE;
    property    LPTNumber     : Byte    read fLPTNumber  write fSetLPTNumber default 1;

  end;

procedure Register;

implementation

var
  fpMemRec         : PMemRec;
  fpPortRec        : PPortRec;
  fpPciRec         : pPciCfg;
  fpRequest        : pPciRequest;
  fIoPrivList      : pByteArray;


constructor TVicHW32.Create(AOwner:TComponent);
begin
  fHardAccess     := TRUE;
  inherited Create (AOwner);
  fOpenDrive      := FALSE;
  hDRV            := 0;
  fMapped         := FALSE;
  fIrqNumber      := 7;
  fMasked := TRUE;
  GetMem(fpMemRec,sizeof(TMemRec));
  GetMem(fpPortRec,sizeof(TPortRec));
  GetMem(fpPciRec,256);
  fpRequest := Pointer(fpPciRec);
  GetMem(fIoPrivList,65536);
  FillChar(fIoPrivList^,$FFFF,0);
  fIoPrivLevel     := 0;

  fLPTs            := 0;
  fLPTBasePort     := $378;
  fLPTNumber       := 1;
  fLPTReadMode     := FALSE;
  
end;

destructor TVicHW32.Destroy;
begin
 if (fOpenDrive) then CloseDriver;
 fOpenDrive      := FALSE;
 hDRV            := 0;
 fMapped         := FALSE;
 fResetAllPermissions();
 inherited Destroy;
 FreeMem(fpMemRec);
 FreeMem(fpPortRec);
 FreeMem(fpPciRec);
 FreeMem(fIoPrivList);
end;

procedure TVicHW32.OpenDriver;
var
  i,p : Word;
begin
  if not fOpenDrive then
  begin
    hDRV:=open('/dev/tvichw',0);
    fOpenDrive := hDRV > 0;
    fHardAccess := TRUE;
    fMasked := TRUE;
    fHandled := 0;

    fLPTs            := 0;
    fLPTBasePort     := $378;
    fLPTNumber       := 1;

    fMapPhysToLinear($00000408,6);

    for i:= 1 to 3 do
    begin

      p:= MemW[$408,i-1];
      if (p<>0) then
      begin
        Inc(fLpts);
        fLptAddresses[fLpts] := p
      end;

    end;

    fUnmapMemory;

    if fLpts >0 then
    begin
      fLPTBasePort := fLPTAddresses[1];
      fLPTNumber   := 1;
      fLptIrqForced:= 0;
      fLPTReadMode := FALSE;
    end;

  end;
end;

procedure TVicHW32.CloseDriver;
begin
  if fOpenDrive then
  begin
    if fMapped then fUnmapMemory;
    if not fMasked then MaskInterrupt;
    __close(hDRV);
  end;
  fOpenDrive  := FALSE;
  fHardAccess := TRUE;
  fMasked := TRUE;
end;

var  __HW32__        : TVicHW32;

procedure TheIsr( Signum: Integer; const Info: TSigInfo; Ignored: Pointer); cdecl;
begin
  Inc(__HW32__.fHandled);
  if Assigned(__HW32__.fOnHwInterrupt) then
    __HW32__.OnHwInterrupt(__HW32__.fIrqNumber);
end;


function TVicHW32.GetHandledCounter: Longword;
begin
  Result := fHandled;
end;

procedure  TVicHW32.UnmaskInterrupt(IrqNumber : Word);
var
 Rec: TIrqRequest;
 SigAct: TSigAction;
 SigSet: TSigSet;
begin
  if fMasked then
  begin
    fHandled := 0;
    fIrqNumber := IrqNumber;
    Rec.IrqNumber := IrqNumber;
    Rec.SignalNumber := LP_SIGNAL;
    Rec.ProcessId := CurrentProcessId;
    Rec.CompPointer:= Self;
    __HW32__ := Self;
    sigemptyset(SigSet);
    SigAct.__sigaction_handler := @TheIsr;
    SigAct.sa_mask := SigSet;
    SigAct.sa_flags := SA_SIGINFO;
    sigaction(LP_SIGNAL, @SigAct, NIL);
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_SET_Irq,SizeOf(TIrqRequest)), @Rec);
  end;
  fMasked := FALSE;
end;

procedure  TVicHW32.MaskInterrupt();
var
 Rec: TIrqRequest;
 SigAct: TSigAction;
 SigSet: TSigSet;
begin
  if not fMasked then
  begin
    Rec.IrqNumber := fIrqNumber;
    Rec.SignalNumber := LP_SIGNAL;
    Rec.ProcessId := CurrentProcessId;
    sigemptyset(SigSet);
    SigAct.__sigaction_handler := Pointer(SIG_DFL);
    SigAct.sa_mask := SigSet;
    SigAct.sa_flags := SA_SIGINFO;
    sigaction(LP_SIGNAL, @SigAct, NIL);
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_FREE_Irq,SizeOf(TIrqRequest)), @Rec);
  end;
  fMasked := TRUE;  
end;


function   TVIcHW32.InterruptCounter: Longword;
begin
  if fOpenDrive then
    Result :=  ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_GET_COUNTER,0), NIL)
  else
    Result := 999;  
end;



function  TVicHW32.fGetPortB(PortAddr: word):Byte;
label hard;
begin
  if not fOpenDrive then begin Result:=$ff; Exit; end;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=0;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_READ_PORT,SizeOf(TPortRec)), fpPortRec);
    Result:=fpPortRec^.DataPort and $000000FF;
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        fHardAccess:=TRUE; goto hard;  // can be done for root only!
      end;
    asm
      mov  dx,PortAddr
      in   al,dx
      mov  Result,al
    end;
  end;
end;

function  TVicHW32.fGetPortW(PortAddr: word):Word;
label hard;
begin
  if not fOpenDrive then begin Result:=$ffff; Exit; end;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=0;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_READ_PORTW,SizeOf(TPortRec)), fpPortRec);
    Result:=fpPortRec^.DataPort;
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        Result := $FFFF; Exit;
      end;
    asm
      mov  dx,PortAddr
      in   ax,dx
      mov  Result,ax
    end;

  end;
end;

function  TVicHW32.fGetPortL(PortAddr: word):Longword;
label hard;
begin
  if not fOpenDrive then begin Result:=$ff; Exit; end;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=0;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_READ_PORTL,SizeOf(TPortRec)), fpPortRec);
    Result:=fpPortRec^.DataPort;
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        fHardAccess:=TRUE; goto hard;  // can be done for root only!
      end;
    asm
      mov  dx,PortAddr
      in   eax,dx
      mov  Result,eax
    end;
  end;

end;

procedure   TVicHW32.fSetPortB(PortAddr : word; nNewValue: Byte);
label hard;
begin
  if not fOpenDrive then Exit;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=nNewValue;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_WRITE_PORT,SizeOf(TPortRec)), fpPortRec);
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        fHardAccess:=TRUE; goto hard;  // can be done for root only!
      end;
    asm
      mov  dx,PortAddr
      mov  al,nNewValue
      out  dx,al
    end;
  end;

end;

procedure   TVicHW32.fSetPortW(PortAddr : word; nNewValue: Word);
label hard;
begin
  if not fOpenDrive then Exit;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=nNewValue;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_WRITE_PORTW,SizeOf(TPortRec)), fpPortRec);
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        fHardAccess:=TRUE; goto hard;  // can be done for root only!
      end;
    asm
      mov  dx,PortAddr
      mov  ax,nNewValue
      out  dx,ax
    end;
  end;
end;

procedure   TVicHW32.fSetPortL(PortAddr : word; nNewValue: Longword);
label hard;
begin
  if not fOpenDrive then Exit;
  hard:
  if fHardAccess then
  begin
    fpPortRec^.PortNumber:=PortAddr;
    fpPortRec^.DataPort:=nNewValue;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_HARD_WRITE_PORTL,SizeOf(TPortRec)), fpPortRec);
  end
  else
  begin
    if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then
      begin
        fHardAccess:=TRUE; goto hard;  // can be done for root only!
      end;
    asm
      mov  dx,PortAddr
      mov  eax,nNewValue
      out  dx,eax
    end;
  end;
end;

procedure   TVicHW32.ReadPortFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then Exit;
  asm
    push edi
    cld
    xor  ecx,ecx
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  edi,Buffer
    rep  insb
    pop  edi
  end;
end;

procedure   TVicHW32.WritePortFIFO( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then Exit;
  asm
    push esi
    cld
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  esi,Buffer
    rep  outsb
    pop  esi
  end;

end;

procedure   TVicHW32.ReadPortWFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
    if not fSetIoPermission(PortAddr) then Exit;
  asm
    push ebp
    push esp
    push edi
    cld
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  edi,Buffer
    rep  insw
    pop  edi
    pop  esp
    pop  ebp
  end;

end;

procedure   TVicHW32.WritePortWFIFO( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then Exit;
  asm
    push esi
    cld
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  esi,Buffer
    rep  outsw
    pop  esi
  end;

end;

procedure   TVicHW32.ReadPortLFIFO ( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then Exit;
  asm
    push edi
    cld
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  edi,Buffer
    rep  insd
    pop  edi
  end;

end;

procedure   TVicHW32.WritePortLFIFO( PortAddr: word; Counter:Longword; Buffer: Pointer);
begin
  if fIoPrivList^[PortAddr] = 0 then
      if not fSetIoPermission(PortAddr) then Exit;
  asm
    push esi
    cld
    mov  dx,PortAddr
    mov  ecx,Counter
    mov  esi,Buffer
    rep  outsd
    pop  esi
  end;

end;

function   TVicHW32.fSetIoPermission( PortAddr : word): Boolean;
begin

  Result := false;
  if (PortAddr >= $400) then
  begin
    if (fIoPrivLevel = 0) then
    begin
      if (iopl(3) <0) then exit;
      fIoPrivLevel := 3;
    end;
    fIoPrivList^[PortAddr] := 1;
    Result := true;
    Exit;
  end;

  if (fIoPrivList^[PortAddr] = 0) then
  begin
    if ( ioperm(PortAddr,4,1) < 0) then exit;
  end;
  fIoPrivList^[PortAddr] := 1;
  Result := true;

end;

procedure    TVicHW32.fResetAllPermissions();
var
  i : word;
begin
  if fIoPrivLevel = 3 then
  begin
    iopl(0);
    fIoPrivLevel := 0;
  end;
  for i:=$400 to $FFF0 do
    if fIoPrivList^[i] = 1 then
    begin
      ioperm(i,4,0);
      fIoPrivList^[i] := 0;
    end;
  fIoPrivList := NIL
end;

function  TVicHW32.fMapPhysToLinear(PhAddr:Longword; PhSize:Longword):Pointer;
var
  m: Longword;
begin
  if not fOpenDrive then begin Result := NIL; Exit; end;
  if fMapped and ((PhAddr<>fMappedArea) or (fMappedSize<PhSize)) then
      fUnmapMemory;
  if not fMapped then
  begin
    fpMemRec^.PhysAddress  := PhAddr;
    m:=fpMemRec^.SizeMemBlock;
    fpMemRec^.SizeMemBlock := PhSize;
    ioctl(hDrv,__IOWR(MAJOR_NUM,_DRV_MAP_MEMORY,16), fpMemRec);
    fMappedArea  := PhAddr;
    fMappedSize  := PhSize;
    fKernelArea  := fpMemRec^.KernelLinAddress;
    fpMemRec^.SizeMemBlock := m;
    fMapped := fKernelArea<>0;
  end;
  Result := Pointer(fKernelArea);
end;

procedure TVicHW32.fUnmapMemory();
begin
  if fOpenDrive and fMapped then
    ioctl(hDrv,__IOWR(MAJOR_NUM,_DRV_UNMAP_MEMORY,4), @fKernelArea);
  fMapped := FALSE;
end;

procedure   TVicHW32.fReadMappedMemory(MemRec: pMemRec);
begin
  if fOpenDrive and fMapped then
  begin
     ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_READ_MEMORY,4),MemRec);
  end;
end;

procedure   TVicHW32.fWriteMappedMemory(MemRec: pMemRec);
begin
  if fOpenDrive then
  begin
     ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_WRITE_MEMORY,4),MemRec);
  end;
end;


function TVicHW32.fGetMemB (BaseAddress:Longword; Offset: Longword):Byte;
var
  Res : Byte;
begin
  Res:=$FF;
  if fOpenDrive then
  begin
    fpMemRec^.MemOffset   := Offset;
    fpMemRec^.SizeMemBlock:= 1;
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
    begin
      fReadMappedMemory(fpMemRec);
      move(fpMemRec^.MemBlock[0],Res,1);
    end
    else
      Res := 0;
  end;
  Result:=Res;
end;

procedure TVicHW32.fSetMemB (BaseAddress:Longword; Offset: Longword; pval:Byte);
begin
  if fOpenDrive then
  begin
    fpMemRec^.MemOffset   := Offset;
    fpMemRec^.SizeMemBlock:= 1;
    fpMemRec^.MemBlock[0] := pval;
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
      fWriteMappedMemory(fpMemRec);
  end;
end;

function TVicHW32.fGetMemW (BaseAddress:Longword; Offset: Longword):Word;
var
  Res : Word;
begin
  Res:=$FF;
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := BaseAddress;
    fpMemRec^.MemOffset   := Offset*2;
    fpMemRec^.SizeMemBlock:= 2;
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
    begin
      fReadMappedMemory(fpMemRec);
      move(fpMemRec^.MemBlock[0],Res,2);
    end
    else
     Res := 0;
  end;
  Result:=Res;
end;

procedure TVicHW32.fSetMemW (BaseAddress:Longword; Offset: Longword; pval:Word);
begin
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := BaseAddress;
    fpMemRec^.MemOffset   := Offset*2;
    fpMemRec^.SizeMemBlock:= 2;
    move(pval,fpMemRec^.MemBlock[0],2);
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
      fWriteMappedMemory(fpMemRec);
  end;
end;

function TVicHW32.fGetMemL (BaseAddress:Longword; Offset: Longword):Longword;
var
  Res : Longword;
begin
  Res:=$FF;
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := BaseAddress;
    fpMemRec^.MemOffset   := Offset*4;
    fpMemRec^.SizeMemBlock:= 4;
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
    begin
      fReadMappedMemory(fpMemRec);
      move(fpMemRec^.MemBlock[0],Res,4);
    end
    else
     Res := 0;
  end;
  Result:=Res;
end;

procedure TVicHW32.fSetMemL (BaseAddress:Longword; Offset: Longword; pval:Longword);
begin
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := BaseAddress;
    fpMemRec^.MemOffset   := Offset*4;
    fpMemRec^.SizeMemBlock:= 4;
    move(pval,fpMemRec^.MemBlock[0],4);
    if ((not fMapped) or (BaseAddress<>fMappedArea)) then
      fKernelArea := Longword(fMapPhysToLinear(BaseAddress,MaxMSize));
    if fMapped then
      fWriteMappedMemory(fpMemRec);
  end;
end;

procedure TVicHW32.ReadPhysicalMemory (PhysAddress:Longword; Offset: Longword; Counter: Longword; Buffer: Pointer);
var
  sz : Longword;
begin
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := PhysAddress;
    fpMemRec^.MemOffset   := Offset;
    fpMemRec^.SizeMemBlock:= Counter;
    if ((not fMapped) or (PhysAddress<>fMappedArea)) then
    begin
      if (OffSet+Counter+1)> MaxMSize then sz := (OffSet+Counter+1)
                                      else sz := MaxMSize;
      fKernelArea := Longword(fMapPhysToLinear(PhysAddress,sz));
    end;
    if fMapped then
    begin
      fReadMappedMemory(fpMemRec);
      move(fpMemRec^.MemBlock[0],Buffer^,Counter);
    end;
  end;
end;

procedure TVicHW32.WritePhysicalMemory (PhysAddress:Longword; Offset: Longword; Counter: Longword; Buffer: Pointer);
var
  sz : Longword;
begin
  if fOpenDrive then
  begin
    fpMemRec^.PhysAddress := PhysAddress;
    fpMemRec^.MemOffset   := Offset;
    fpMemRec^.SizeMemBlock:= Counter;
    if ((not fMapped) or (PhysAddress<>fMappedArea)) then
    begin
      if (OffSet+Counter+1)> MaxMSize then sz := (OffSet+Counter+1)
                                      else sz := MaxMSize;
      fKernelArea := Longword(fMapPhysToLinear(PhysAddress,sz));
    end;
    if fMapped then
    begin
      move(Buffer^, fpMemRec^.MemBlock[0],Counter);
      fWriteMappedMemory(fpMemRec);
    end;
  end;
end;

procedure   TVicHW32.fSetHardAccess(Parm : Boolean);
begin
 if (iopl(3) <0) then fHardAccess:=TRUE
                 else fHardAccess:=Parm;
end;

function TVicHW32.PciDeviceInfo(bus,dev,func : Word; Info : pPciCfg) : Boolean;
begin
  Result := FALSE;
  if fOpenDrive then
  begin
    Info.vendorID := Bus;
    Info.deviceID := Word((word(dev and 31) shl 3) or word(func and 7));
    Info.command_reg := 0;
    Info.status_reg  := 64;
    Result := (ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_READ_PCI_CONFIG,64), Info)<>0);
  end;

end;

function TVicHW32.PciGetByte(bus,dev,func: Word; Offset: Word) : Byte;
begin
  Result := 0;
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length  := 1;
    Result := Byte(ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_READ_PCI_CONFIG,64), fpPciRec));
  end;
end;

function TVicHW32.PciGetWord(bus,dev,func: Word; Offset: Word) : Word;
begin
  Result := 0;
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length  := 2;
    Result := Word(ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_READ_PCI_CONFIG,64), fpPciRec));
  end;
end;

function TVicHW32.PciGetDword(bus,dev,func: Word; Offset: Word) : Longword;
begin
  Result := 0;
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length  := 4;
    Result := ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_READ_PCI_CONFIG,64), fpPciRec);
  end;
end;

procedure TVicHW32.PciSetByte(bus,dev,func: Word; Offset: Word; nByte:Byte);
begin
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length := 1;
    fpRequest^.val    := nByte;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_WRITE_PCI_CONFIG,64), fpPciRec);
  end;

end;

procedure TVicHW32.PciSetWord(bus,dev,func: Word; Offset: Word; nWord:Word);
begin
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length := 2;
    fpRequest^.val    := nWord;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_WRITE_PCI_CONFIG,64), fpPciRec);
  end;

end;
procedure TVicHW32.PciSetDword(bus,dev,func: Word; Offset: Word; nDWord:DWord);
begin
  if fOpenDrive then
  begin
    fpRequest^.BusNum := Bus;
    fpRequest^.DevFun := Word((word(dev and 31) shl 3) or word(func and 7));
    fpRequest^.Offset := Offset;
    fpRequest^.length  := 4;
    fpRequest^.val    := nDWord;
    ioctl(hDRV,__IOWR(MAJOR_NUM,_DRV_WRITE_PCI_CONFIG,64), fpPciRec);
  end;

end;

procedure TVicHW32.fSetLPTIRQ(nNewValue  : Boolean);
var
 data : Byte;
begin
  if fOpenDrive then
  begin
    if nNewValue then
        data := fDataWritePorts[2] or $10
    else
        data := fDataWritePorts[2] and $EF;
    fSetPortB(fLPTBasePort+2,data);
  end;
end;

procedure TVicHW32.fSetLPTNumber(nNewValue  : Byte);
begin
  if fOpenDrive and (nNewValue <= fLPTs) then
  begin
    fLPTBasePort := fLPTAddresses[nNewValue];
    fDataReadPorts[0]  := fGetPortB(fLPTBasePort);
    fDataReadPorts[1]  := fGetPortB(fLPTBasePort+1);
    fDataReadPorts[2]  := fGetPortB(fLPTBasePort+2);
    fDataWritePorts[0] := fGetPortB(fLPTBasePort);
    fDataWritePorts[1] := fGetPortB(fLPTBasePort+1);
    fDataWritePorts[2] := fGetPortB(fLPTBasePort+2);
    fLptIrqForced      := fDataWritePorts[2] and $10;
    fLPTNumber := nNewValue;
  end;
end;

function TVicHW32.fGetPin(nPin : Byte) : Boolean;
var
  data, ofs: Byte;
begin
  Result := FALSE;
  if fOpenDrive and (nPin <= 17) and ( nPin > 0) then
  begin
    ofs := PinsPort[nPin];
    data := fGetPortB(fLPTBasePort+ofs);
    fDataReadPorts[ofs] := data;
    Result := (data and MaskPins[nPin]) <> 0;
    if Negative[nPin] then Result := not Result;
  end;
end;

procedure TVicHW32.fSetPin(nPin : Byte; bNewValue : Boolean);
var
  data, ofs: Byte;
begin
  if fOpenDrive and (nPin <= 17) and ( nPin > 0) then
  begin
    ofs    := PinsPort[nPin];
    data   := fDataWritePorts[ofs];
    if (bNewValue <> Negative[nPin]) then
      data := data or MaskPins[nPin]
    else
      data := data and (not MaskPins[nPin]);
    fSetPortB(fLPTBasePort+ofs,data);
    fDataWritePorts[ofs] := data;
  end;
end;

procedure   TVicHW32.fSetLPTReadMode(readmode : Boolean);
begin
  if not fOpenDrive then Exit;
  if readmode then
    fDataWritePorts[2]  := fDataWritePorts[2] or $20
  else
    fDataWritePorts[2]  := fDataWritePorts[2] and $DF;
  fSetPortB(fLPTBasePort+2,fDataWritePorts[2]);
  fLPTReadMode := readmode;
end;

function TVicHW32.fGetLPTAckwl(): Boolean;
begin
  Result := not fGetPin(10);
end;

function TVicHW32.fGetLPTBusy : Boolean;
begin
  Result:= fGetPin(11);
end;

function TVicHW32.fGetLPTPaperEnd : Boolean;
begin
  Result:= fGetPin(12);
end;

function TVicHW32.fGetLPTSlct: Boolean;
begin
  Result:=  not fGetPin(13);
end;

function TVicHW32.fGetLPTError : Boolean;
begin
  Result:=  not fGetPin(15);
end;

procedure   TVicHW32.LPTStrobe;
var
  i : Word;
begin
  fSetPin(1,FALSE);
  for i := 0 to 1000 do
  fSetPin(1,TRUE);
end;

procedure TVicHW32.LPTAutofd(Flag : Boolean);
begin
  fSetPin(14, not Flag);
end;

procedure TVicHW32.LPTInit;
var
  i : Word;
begin
  fSetPin(16,FALSE);
  for i := 0 to 1000 do
  fSetPin(16,TRUE);
end;

procedure TVicHW32.LPTSlctIn;
var
  data : Byte;
begin
  fSetPin(16,FALSE);
  data := fDataWritePorts[2] and $f7;
  fDataWritePorts[2] := data;
  fSetPortB(fLPTBasePort+2,data);
end;


function    TVicHW32.GetDebugCode: Longword;
begin
  Result := Longword(fIoPrivList);
end;


procedure Register;
begin
  RegisterComponents('Drivers', [TVicHw32]);
end;


end.
