unit IrqFm;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, IdGlobal, Libc,
  HW_32, QTypes, QExtCtrls, QComCtrls;

type
  TIrqForm = class(TForm)
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label36: TLabel;
    L_Gen: TLabel;
    L_Han: TLabel;
    L_Per: TLabel;
    B_LPTGen: TButton;
    Timer1: TTimer;
    C_Irq: TCheckBox;
    L_Irq: TListBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure B_ExitClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure B_LPTGenClick(Sender: TObject);
    procedure C_IrqClick(Sender: TObject);
    procedure HW32HwInterrupt(IrqNumber: Word);
    procedure L_IrqClick(Sender: TObject);
    procedure Label1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowButtons;
  end;

var
  IrqForm: TIrqForm;


implementation

uses MainFm;

{$R *.xfm}

var

  HW32           : TVicHW32;
  IrqN           : Byte;

  IRQHandled     : Longint;
  OldTime, CurrTime : TDateTime;
  Period         : Double;

  nom_pad        : Integer;

procedure TIrqForm.ShowButtons;
begin

  B_LPTGen.Enabled := (IrqN = 7);
  L_Irq.Enabled := not C_Irq.Checked;
end;

procedure TIrqForm.HW32HwInterrupt(IrqNumber: Word);
var
  a,b : double;
begin
  if IRQHandled = 0 then   OldTime := Now;
  Inc(IRQHandled);
  CurrTime:= Now;
  a:=IRQHandled;
  b:=Abs(DateTimeToTimeStamp(CurrTime).Time - DateTimeToTimeStamp(OldTime).Time);
  if IRQHandled>1 then period:= b/(a-1);
  IrqN := IrqNumber;
end;


procedure TIrqForm.FormCreate(Sender: TObject);
var
 s   : array[0..20] of String;
 s1  : String;
 F   : TextFile;
 i,n,nIrq : Integer;
begin
  HW32 := MainForm.HW32;
  IrqN := 0;
  nom_pad := 0;
  HW32.OnHwInterrupt := HW32HwInterrupt;
  ShowButtons;

  L_Irq.Clear;
  AssignFile(F,'/proc/interrupts');
  ReSet(F);
  n:= 0;
  for i:= 0 to 20 do s[i] := '';
  while not eof(F) do
  begin
    ReadLn(F,s1);
    Val(Copy(s1,1,3),nIrq,i);
    if (i = 0) then
    begin
      if nIrq>n then n:= nIrq;
      Delete(s1,6,20);
      s[nIrq] := s1;
    end;
  end;
  CloseFile(F);

  for i:= 0 to n do
  begin
    if s[i]='' then
    begin
      Str(i:3,s1);
      s1 := s1+ ': XT-PIC  = free';
      case i of
        1 : s1 := s1 + ', assume keyboard';
        3 : s1 := s1 + ', assume COM2';
        4 : s1 := s1 + ', assume COM1';
        5 : s1 := s1 + ', assume LPT2';
        7 : s1 := s1 + ', assume LPT1';
      end;
      s[i]:=s1 + ' =';
    end;
    L_Irq.Items.Add(s[i]);
  end;

  L_Irq.ItemIndex := 7;

end;

procedure TIrqForm.B_ExitClick(Sender: TObject);
begin
  Close;
end;

procedure TIrqForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
end;


procedure TIrqForm.Timer1Timer(Sender: TObject);
var
  s : String;
begin
  if (HW32 <> NIL) and (IrqN>0) then
  begin
    L_Gen.Caption := IntToStr(HW32.InterruptCounter(IrqN));
    L_Han.Caption := IntToStr(IRQHandled);
    Str(period:8:4,s);
    L_Per.Caption:=s;
  end;
end;

procedure TIrqForm.B_LPTGenClick(Sender: TObject);
var
  nPort : Word;
begin
      // for this test you must create an electrical connection between
      // the pin10(Ack line) and Pin 2..9 by the chunk of wire.
      // In this case, when we will write ff/00 seuence to the data
      // register then it will provoke an interruptions

      nPort := HW32.LPTBasePort;

      // enable interuptions from LPT
      HW32.LptIrqEnabled := TRUE;
      // create impulse
      HW32.Port[nPort] := $00;  // write 00 to lpt base port
      HW32.Port[nPort] := $FF;  // write ff to lpt base port

end;

procedure TIrqForm.C_IrqClick(Sender: TObject);
begin
 if C_Irq.Checked then
  begin
    IRQHandled  := 0;

    HW32.UnmaskInterrupt(IrqN);

    OldTime:=Now;
  end
  else begin
         HW32.MaskInterrupt(IrqN);
       end;

  ShowButtons;
end;


procedure TIrqForm.L_IrqClick(Sender: TObject);
begin
  IrqN := L_Irq.ItemIndex;
  C_Irq.Caption := 'unmask selected Irq'+IntToStr(IrqN);
  ShowButtons;
end;

procedure TIrqForm.Label1Click(Sender: TObject);
begin
  if nom_pad<(__SI_PAD_SIZE-1) then Inc(nom_pad) else nom_pad := 0;
end;


initialization
  HW32 := NIL;
  IRQHandled := 0;
end.
