unit MainFm;

interface

uses
  SysUtils,  Types,  Classes,  Variants,  QGraphics,
  QControls, QForms, QDialogs, QStdCtrls, QExtCtrls,
  HW_32, HW_Types, PciFm, PortFm, MemFm, IrqFm, LptFm;


type
  TMainForm = class(TForm)
    B_Exit: TButton;
    Panel1: TPanel;
    Label4: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    HW32: TVicHw32;
    R_Select: TRadioGroup;
    B_OpenClose: TButton;
    procedure B_ExitClick(Sender: TObject);
    procedure B_OpenCloseClick(Sender: TObject);
    procedure R_SelectClick(Sender: TObject);
    procedure LinkForms(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.xfm}

Var
  CurrForm: TForm;

procedure TMainForm.B_ExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.B_OpenCloseClick(Sender: TObject);
begin
  if not HW32.ActiveHW then
  begin
    HW32.OpenDriver;
    CurrForm := NIL;
    if not HW32.ActiveHW then
    begin
      ShowMessage('Can''t open the driver!');
      Exit;
    end;
    B_OpenClose.Caption := 'Close TVicHW32';
    R_Select.Enabled := TRUE;
  end
  else
  begin
    HW32.CloseDriver;
    if CurrForm<>NIL then
    begin
      CurrForm.Hide;
      CurrForm := NIL;
    end;
    R_Select.ItemIndex := 0;
    R_Select.Enabled := FALSE;
    B_OpenClose.Caption := 'Open TVicHW32';
  end;

end;

procedure TMainForm.R_SelectClick(Sender: TObject);
begin
  if CurrForm<>NIL then
  begin
    CurrForm.Hide;
    CurrForm.Close;
    CurrForm.Free;
  end;
  CurrForm := NIL;
  if HW32.ActiveHW then
  case R_Select.ItemIndex of
    1: begin
         PortForm  := TPortForm.Create(MainForm);
         CurrForm  := PortForm;
         LinkForms(Sender);
         PortForm.Show;
       end;
    2: begin
         MemForm  := TMemForm.Create(MainForm);
         CurrForm := MemForm;
         LinkForms(Sender);
         MemForm.Show;
       end;
    3: begin
         LptForm  := TLptForm.Create(MainForm);
         CurrForm := LptForm;
         LinkForms(Sender);
         LptForm.Show;
       end;
    4: begin
         PciForm  := TPciForm.Create(MainForm);
         CurrForm := PciForm;
         LinkForms(Sender);
         PciForm.Show;
    end;
    5: begin
         IrqForm  := TIrqForm.Create(MainForm);
         CurrForm := IrqForm;
         LinkForms(Sender);
         IrqForm.Show;
    end;

  end;

end;

procedure TMainForm.LinkForms(Sender: TObject);
begin
  CurrForm.Left := Left+Width+5;
  CurrForm.Top  := MainForm.Top-23;

end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Left := 40;
  Top  := 40;
end;

end.
