unit PciFm;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QGrids,
  HW_32, HW_Types;
type
  TPciForm = class(TForm)
    Label44: TLabel;
    L_AllPCI: TLabel;
    G_PCI: TStringGrid;
    L_PCI: TListBox;
    B_AllPci: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure B_AllPciClick(Sender: TObject);
    procedure B_GetPciClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  PciForm: TPciForm;

implementation

uses MainFm;

{$R *.xfm}

var
  HW32 : TVicHW32;
  w,h  : Word;

procedure TPciForm.FormCreate(Sender: TObject);
begin
  HW32 := MainForm.HW32;
  w := Width;
  h := Height;
  with G_PCI do
  begin
    Cells[0,0] := 'Vendor ID';
    Cells[0,1] := 'DeviceID';
    Cells[0,2] := 'Command';
    Cells[0,3] := 'Status';
    Cells[0,4] := 'RevisionID';
    Cells[0,5] := 'ProgIf';
    Cells[0,6] := 'SubClass';
    Cells[0,7] := 'BaseClass';
    Cells[0,8] := 'CacheLineSize';
    Cells[0,9] := 'LatencyTimer';
    Cells[0,10] := 'HeaderType';
    Cells[0,11] := 'BIST';
    Cells[0,12] := 'BaseAddresses[0]';
    Cells[0,13] := 'BaseAddresses[1]';
    Cells[0,14] := 'BaseAddresses[2]';
    Cells[0,15] := 'BaseAddresses[3]';
    Cells[0,16] := 'BaseAddresses[4]';
    Cells[0,17] := 'BaseAddresses[5]';
    Cells[0,18] := 'CardBusCISPtr';
    Cells[0,19] := 'SubsystemVendorID';
    Cells[0,20] := 'SubsystemID';
    Cells[0,21] := 'ROMBaseAddress';
    Cells[0,22] := 'InterruptLine';
    Cells[0,23] := 'InterruptPin';
    Cells[0,24] := 'MinimumGrant';
    Cells[0,25] := 'MaximumLatency';
  end;

end;

procedure TPciForm.FormResize(Sender: TObject);
begin
  Width := w;
  Height := h;
  MainForm.LinkForms(Sender);
end;

procedure TPciForm.B_AllPciClick(Sender: TObject);
var buses,n,bus,dev,func : Longword;
    Info                 : TPciCfg;
    s                    : String;
begin

    B_AllPci.Enabled := FALSE;

    buses := 15;

    n:=0;
    L_PCI.Clear;
    for bus := 0 to buses do
    begin
      for dev := 0 to 31 do
      begin
        for func := 0 to 7 do
        begin

          if HW32.PciDeviceInfo(bus,dev,func,@Info) then
          begin
            Inc(n);
            s :=  IntToHex(bus,2)+'/'+IntToHex(dev,2)+'/'+IntToHex(func,2)+
             '  '+IntToHex(Info.ClassCode,2)+'  '+IntToHex(info.VendorID,4)+
            '   '+IntToHex(Info.DeviceID,4);
            s:=s+'  '+IntToHex(info.header_type,2)+'=>';

            case (info.header_type and $7F) of
                   $00:  // non-bridge
                   begin
                     s:=s+'non-bridge ';
                   end;
                   $01:  // bridge
                   begin
                     s:=s+'bridge     ';
                   end;
                   $02: // CardBus
                   begin
                     s:=s+'CardBus    ';
                   end;
            end;

            L_PCI.Items.Add(s);
          end;

        end;
      end;
      L_AllPCI.caption:='=> '+IntToStr(n);
      if n>0 then L_PCI.ItemIndex := n;
  end;

  B_AllPci.Enabled := True;
  L_PCI.ItemIndex := n-1;


end;

procedure TPciForm.B_GetPciClick(Sender: TObject);
var s            : String;
    bus,dev,func : Longword;
    Info         : TPciCfg;
    i            : Integer;
    HeaderAsWord : pWordArray;
begin
  if (L_PCI.ItemIndex >= 0) and HW32.ActiveHW then
  begin

    s := L_PCI.Items[L_PCI.ItemIndex];
    bus := StrToInt('$'+Copy(s,1,2));
    dev := StrToInt('$'+Copy(s,4,2));
    func:= StrToInt('$'+Copy(s,7,2));

    // we can use PciDeviceInfo() method to obtain the Pci header
    // but we want demonstarte how to use PciGetWord() method instead

    HeaderAsWord := @Info;

    for i:= 0 to 31 do
       HeaderAsWord^[i] := HW32.PciGetWord(bus,dev,func,i*2);

    with  G_PCI, Info, Info.NonBridge do
    begin

      Cells[1,0] := IntToHex(VendorID,4);
      Cells[1,1] := IntToHex(DeviceID,4);
      Cells[1,2] := IntToHex(Command_reg,4);

      Cells[1,3] := IntToHex(Status_reg,4);

      Cells[1,4] := IntToHex(RevisionID,2);
      Cells[1,5] := IntToHex(ProgIf,2);
      Cells[1,6] := IntToHex(SubClass,2);
      Cells[1,7] := IntToHex(ClassCode,2);
      Cells[1,8] := IntToHex(CacheLine_Size,2);
      Cells[1,9] := IntToHex(Latency,2);
      Cells[1,10] := IntToHex(Header_Type,2);
      Cells[1,11] := IntToHex(BIST,2);
      Cells[1,12] := IntToHex(Base_Address0,8);
      Cells[1,13] := IntToHex(Base_Address1,8);
      Cells[1,14] := IntToHex(Base_Address2,8);
      Cells[1,15] := IntToHex(Base_Address3,8);
      Cells[1,16] := IntToHex(Base_Address4,8);
      Cells[1,17] := IntToHex(Base_Address5,8);
      Cells[1,18] := IntToHex(CardBus_CIS,8);
      Cells[1,19] := IntToHex(Subsystem_VendorID,4);
      Cells[1,20] := IntToHex(Subsystem_DeviceID,4);
      Cells[1,21] := IntToHex(expansion_ROM,8);
      Cells[1,22] := IntToHex(Interrupt_Line,2);
      Cells[1,23] := IntToHex(Interrupt_Pin,2);
      Cells[1,24] := IntToHex(Min_Grant,2);
      Cells[1,25] := IntToHex(Max_Latency,2);
    end;

  end;

end;

procedure TPciForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
end;

end.
