unit PortFm;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QExtCtrls, QStdCtrls, QComCtrls, QGrids, IdGlobal,
  HW_32;

type
  TPortForm = class(TForm)
    C_Hard: TCheckBox;
    GroupBox1: TGroupBox;
    B_Read: TButton;
    B_ReadAll: TButton;
    B_Write: TButton;
    B_WriteAll: TButton;
    GR: TStringGrid;
    GW: TStringGrid;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    B_ReadFIFO: TButton;
    S_Elements: TSpinEdit;
    ListFifo: TListBox;
    E_PortAddr: TEdit;
    R_PortWide: TRadioGroup;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    Label7: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    L_Test1: TLabel;
    L_Test2: TLabel;
    L_Test4: TLabel;
    E_Cycles: TEdit;
    B_Test: TButton;
    L_Ports: TListBox;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ReadPorts(First,Last: Integer);
    procedure WritePorts(First,Last : Integer);
    procedure B_ReadClick(Sender: TObject);
    procedure B_ReadAllClick(Sender: TObject);
    procedure B_WriteClick(Sender: TObject);
    procedure B_WriteAllClick(Sender: TObject);
    procedure C_HardClick(Sender: TObject);
    procedure B_TestClick(Sender: TObject);
    procedure B_ReadFIFOClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  PortForm: TPortForm;

implementation

uses MainFm;

{$R *.xfm}

const
  MaxPorts = 4;
var
  HW32  : TVicHW32;

var
  NumPortCycles : Longint = 100000;
  DataByte      : Byte;
  Cycle         : Longint;
  Buf1          : array[0..65535] of Byte;

procedure TPortForm.ReadPorts(First,Last: Integer);
var
  PortRSel : Longword;
  NomRSel  : Byte;
begin

 with GR do
 begin

   for NomRSel:=First to Last do
   begin
     PortRSel := 0;
     if Cells[1,NomRSel]<>'' then
       PortRSel:=StrToInt('$'+Cells[1,NomRSel]);
     Cells[1,NomRSel]:=IntToHex(PortRSel,4);
     if (PortRSel<>0 ) then
     begin
       case R_PortWide.ItemIndex of
           0: Cells[2,NomRSel]:=IntToHex(HW32.Port[PortRSel],2);
           1: Cells[2,NomRSel]:=IntToHex(HW32.PortW[PortRSel],4);
           2: Cells[2,NomRSel]:=IntToHex(HW32.PortL[PortRSel],8);
       end;
     end;
   end;
 end;
end;

procedure TPortForm.WritePorts(First,Last : Integer);
var
  PortRSel, NomRSel : Word;
  PortData : Longword;
begin
 with GW do
 begin
   for NomRSel := First to Last do
   begin

     PortRSel := 0;
     PortData := 0;
     if Cells[1,NomRSel]<>'' then
     begin
       PortRSel:=StrToInt('$'+Cells[1,NomRSel]);
       if Cells[2,NomRSel]<>'' then
         PortData:=StrToInt('$'+Cells[2,NomRSel]);
     end;
     Cells[1,NomRSel]:=IntToHex(PortRSel,4);
     Cells[2,NomRSel]:=IntToHex(PortData,(R_PortWide.ItemIndex+1)*2);

     if (PortRSel<>0) then
     begin
       case R_PortWide.ItemIndex of
        0: begin
             Cells[2,NomRSel]:=IntToHex(PortData,2);
             HW32.Port[PortRSel] := Byte(PortData);
           end;
        1: begin
             Cells[2,NomRSel]:=IntToHex(PortData,4);
             HW32.PortW[PortRSel] := Word(PortData);
           end;
        2: begin
             Cells[2,NomRSel]:=IntToHex(PortData,8);
             HW32.PortW[PortRSel] := PortData;
           end;
       end;
     end;
   end;
 end;

end;

procedure TPortForm.FormCreate(Sender: TObject);
var
  s1  : String;
  F   : TextFile;
  i   : Integer;

begin

  L_Ports.Clear;
  AssignFile(F,'/proc/ioports');
  ReSet(F);
  while not eof(F) do
  begin
    ReadLn(F,s1);
    L_Ports.Items.Add(s1);
  end;
  CloseFile(F);


  HW32 := MainForm.HW32;

  for i:= 1 to 2 do
  begin
    GR.ColWidths[i] := (GR.Width-GR.DefaultColWidth - 6) div 2;
    GW.ColWidths[i] := (GW.Width-GW.DefaultColWidth - 6) div 2;
  end;
  for i:= 1 to MaxPorts do
  begin
    GR.RowHeights[i] := (GR.Height-GR.DefaultRowHeight - 8) div 4;
    GW.RowHeights[i] := (GW.Height-GW.DefaultRowHeight - 8) div 4;
  end;


  for i:=1 to MaxPorts do
  begin
    with GW do
    begin
      Cells[0,i]:=IntToStr(i);
      Cells[1,0]:='Port'; Cells[2,0]:='Value';
      if i<4 then Cells[1,i]:= IntToHex($377+i,4);
      Cells[2,i]:= '';
    end;
    with GR do
    begin
      Cells[0,i]:=IntToStr(i);
      Cells[1,0]:='Port'; Cells[2,0]:='Value';
      if i<4 then Cells[1,i]:= IntToHex($377+i,4);
    end;
  end;

end;
procedure TPortForm.B_ReadClick(Sender: TObject);
begin
  ReadPorts(GR.Row,GR.Row);
end;

procedure TPortForm.B_ReadAllClick(Sender: TObject);
begin
  ReadPorts(1,MaxPorts);
end;

procedure TPortForm.B_WriteClick(Sender: TObject);
begin
  WritePorts(GW.Row,GW.Row);
end;

procedure TPortForm.B_WriteAllClick(Sender: TObject);
begin
  WritePorts(1,MaxPorts);
end;
procedure TPortForm.C_HardClick(Sender: TObject);
begin
  HW32.HardAccess := C_Hard.Checked;
  C_Hard.Checked  := HW32.HardAccess;
end;

procedure TPortForm.B_TestClick(Sender: TObject);
var i : Longint;
    pRec   : pByteArray;
    a,b,c  : Double;
    s    : String;
    PortAddr : Word;
begin

  B_Test.Enabled := FALSE;

  Val(E_Cycles.text,NumPortCycles,Cycle);
  B_Test.Enabled := FALSE;
  PortAddr := StrToInt('$'+E_PortAddr.text);

  Screen.Cursor := crHourGlass;

  // "Hard" access port I/O mode

  HW32.HardAccess := TRUE;
  a := GetTickCount; c:=NumPortCycles;
  for i := 1 to NumPortCycles do DataByte := HW32.Port[PortAddr];
  b:=GetTickCount; Str((b-a)*1000.0/c : 5:2,s);
  L_Test1.Caption:= s+ ' microsec';

  // "Soft" access port I/O mode

  HW32.HardAccess := FALSE;
  a:= GetTickCount;
  for i := 1 to NumPortCycles do DataByte := HW32.Port[PortAddr];
  b:=GetTickCount; Str((b-a)*1000.0/c : 5:2,s);
  L_Test2.Caption:= s+ ' microsec';

 // with ReadPortFIFO() method

  GetMem(pRec,NumPortCycles);

  a := GetTickCount;
  HW32.ReadPortFIFO(PortAddr,NumPortCycles,pRec);
  b:=GetTickCount; Str((b-a)*1000.0/c : 5:2,s);
  L_Test4.Caption:= s+ ' microsec';

  FreeMem(pRec);


  HW32.HardAccess := C_Hard.Checked;
  Screen.Cursor := crDefault;

  B_Test.Enabled := TRUE;

end;

procedure TPortForm.B_ReadFIFOClick(Sender: TObject);
var
  i           : Word;
  NumPorts,nBytes    : Longword;
  PortAddr    : Word;
  s           : String;
begin

  NumPorts:=S_Elements.Value;

  PortAddr := StrToInt('$'+E_PortAddr.text);
  E_PortAddr.Text:=IntToHex(PortAddr,4);

  case R_PortWide.ItemIndex of
    0 : begin nBytes := 1*NumPorts; HW32.ReadPortFIFO(PortAddr,NumPorts,@Buf1);  end;
    1 : begin nBytes := 2*NumPorts; HW32.ReadPortWFIFO(PortAddr,NumPorts,@Buf1); end;
    2 : begin nBytes := 4*NumPorts; HW32.ReadPortLFIFO(PortAddr,NumPorts,@Buf1); end;
    else
        begin nBytes := 1*NumPorts; HW32.ReadPortFIFO(PortAddr,NumPorts,@Buf1);  end;
  end;
  ListFifo.Clear;

  s:='';
  for i:=1 to nBytes do
  begin
    s:=s+IntToHex(Buf1[i],2);
    if (Length(s)>31) or (i=nBytes) then
    begin
      ListFifo.Items.Add(s);
      s:='';
    end;
  end;

end;

procedure TPortForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
end;


end.
